/* Copyright 2018 Carlos Filoteo
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "drashna.h"
#include QMK_KEYBOARD_H

#define LT3_TAB LT(3, KC_TAB)

// clang-format off
#define LAYOUT_ut472_base( \
    K01, K02, K03, K04, K05, K06, K07, K08, K09, K0A, \
    K11, K12, K13, K14, K15, K16, K17, K18, K19, K1A, \
    K21, K22, K23, K24, K25, K26, K27, K28, K29, K2A  \
  ) \
    LAYOUT_wrapper( \
    KC_ESC,     K01,      K02,     K03,     K04,        K05,                K06,           K07,     K08,        K09,   K0A,    BSPCDEL, \
    KC_TAB,     K11,      K12,     K13,     K14,        K15,                K16,           K17,     K18,        K19,   K1A,    KC_ENT, \
    KC_LSFT,    K21,      K22,     K23,     K24,        K25,                K26,           K27,     K28,        K29,   K2A,    KC_QUOT, \
    KC_LCTL,    LGUI(0),  KC_LALT, KC_LEAD, MO(_LOWER), LT(_NAV, KC_SPC),   MO(_RAISE),    KC_LEFT, KC_DOWN,    KC_UP, KC_RGHT \
    )

/* Re-pass though to allow templates to be used */
#define LAYOUT_ut472_base_wrapper(...)       LAYOUT_ut472_base(__VA_ARGS__)

const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {

    [_QWERTY] = LAYOUT_ut472_base_wrapper(
        _________________QWERTY_L1_________________, _________________QWERTY_R1_________________,
        _________________QWERTY_L2_________________, _________________QWERTY_R2_________________,
        _________________QWERTY_L3_________________, _________________QWERTY_R3_________________
    ),

    [_WORKMAN] = LAYOUT_ut472_base_wrapper(
        _________________WORKMAN_L1________________, _________________WORKMAN_R1________________,
        _________________WORKMAN_L2________________, _________________WORKMAN_R2________________,
        _________________WORKMAN_L3________________, _________________WORKMAN_R3________________
    ),

    [_LOWER] = LAYOUT_ut472_base_wrapper(
        _________________LOWER_L1__________________, _________________LOWER_R1__________________,
        _________________LOWER_L2__________________, _________________LOWER_R2__________________,
        _________________LOWER_L3__________________, _________________LOWER_R3__________________
    ),
    [_RAISE] = LAYOUT_ut472_base_wrapper(
        _________________RAISE_L1__________________, _________________RAISE_R1__________________,
        _________________RAISE_L2__________________, _________________RAISE_R2__________________,
        _________________RAISE_L3__________________, _________________RAISE_R3__________________
    ),
    [_ADJUST] = LAYOUT_ut472_base_wrapper(
        _________________ADJUST_L1_________________, _________________ADJUST_R1_________________,
        _________________ADJUST_L2_________________, _________________ADJUST_R2_________________,
        _________________ADJUST_L3_________________, _________________ADJUST_R3_________________
    ),
    [_NAV] = LAYOUT_ut472_base_wrapper(
        ___________________NAV_L1__________________, ___________________NAV_R1__________________,
        ___________________NAV_L2__________________, ___________________NAV_R2__________________,
        ___________________NAV_L3__________________, ___________________NAV_R3__________________
    ),
//     [_LAYERINDEX] = LAYOUT_wrapper(
//       _______, _______, _______, _______, _______, _______,                                       _______, _______, _______, _______, _______, _______,
//       _______, _______, _______, _______, _______, _______,                                       _______, _______, _______, _______, _______, _______,
//       _______, _______, _______, _______, _______, _______, _______, _______,   _______, _______, _______, _______, _______, _______, _______, _______,
//                                  _______, _______, _______, _______, _______,   _______, _______, _______, _______, _______
//     ),
};
// clang-format on


// const uint16_t PROGMEM keymaps[][MATRIX_ROWS][MATRIX_COLS] = {

//   /* Base Layer
//    * ,-------------------------------------------------------------------------.
//    * | Esc |  Q  |  W  |  E  |  R  |  T  |  Y  |  U  |  I  |  O  |  P  |Bspace |
//    * |-------------------------------------------------------------------------+
//    * |Tab/L3|  A  |  S  |  D  |  F  |  G  |  H  |  J  |  K  |  L  |  ;  |   '  |
//    * |-------------------------------------------------------------------------+
//    * | Shift |  Z  |  X  |  C  |  V  |  B  |  N  |  M  |  ,  |  .  |  /  |Sh/En|
//    * |-------------------------------------------------------------------------+
//    * | Ctrl| Alt | Gui | App |  L2  |   Space   |  L1  | Left| Down|  Up |Right|
//    * `-------------------------------------------------------------------------'
//    */

// LAYOUT(
//   KC_ESC,  KC_Q,    KC_W,    KC_E,    KC_R,    KC_T,    KC_Y,    KC_U,    KC_I,    KC_O,    KC_P,    KC_BSPC,
//   LT3_TAB, KC_A,    KC_S,    KC_D,    KC_F,    KC_G,    KC_H,    KC_J,    KC_K,    KC_L,    KC_SCLN, KC_QUOT,
//   KC_LSFT, KC_Z,    KC_X,    KC_C,    KC_V,    KC_B,    KC_N,    KC_M,    KC_COMM, KC_DOT,  KC_SLSH, KC_SFTENT,
//   KC_LCTL, KC_LALT, KC_LGUI, KC_APP,  MO(2),      KC_SPC,        MO(1),  KC_LEFT, KC_DOWN, KC_UP,   KC_RGHT
// ),

//   /* FN Layer 1
//    * ,-------------------------------------------------------------------------.
//    * | ` ~ |  1  |  2  |  3  |  4  |  5  |  6  |  7  |  8  |  9  |  0  | Delete|
//    * |-------------------------------------------------------------------------+
//    * |      |     |     |     |     |     |     |  -  |  =  |  [  |  ]  |  \   |
//    * |-------------------------------------------------------------------------+
//    * |       | F11 | F12 | F13 | F14 | F15 | F16 | F17 | F18 | F19 | F20 |     |
//    * |-------------------------------------------------------------------------+
//    * |     |     |     |Capsl|      |          |       | Home| PgDn| PgUp| End |
//    * `-------------------------------------------------------------------------'
//    */

// LAYOUT( /* Right */
//   KC_GRV,  KC_1,    KC_2,    KC_3,    KC_4,    KC_5,    KC_6,    KC_7,    KC_8,    KC_9,    KC_0,    KC_DELETE,
//   _______, RGB_TOG, RGB_MOD, RGB_VAI, RGB_VAD, _______, _______, KC_MINS, KC_EQL,  KC_LBRC, KC_RBRC, KC_BSLS,
//   _______, KC_F11,  KC_F12,  KC_F13,  KC_F14,  KC_F15,  KC_F16,  KC_F17,  KC_F18,  KC_F19,  KC_F20,  _______,
//   _______, _______, _______, KC_CAPS, _______,     _______,      _______, KC_HOME, KC_PGDN, KC_PGUP, KC_END
// ),

//    FN Layer 2
//    * ,-------------------------------------------------------------------------.
//    * |   ~  |  !  |  @  |  #  |  $  |  %  |  ^  |  &  |  *  |  (  |  )  |Delete|
//    * |-------------------------------------------------------------------------+
//    * |      |     |     |     |     |     |     |  _  |  +  |  { |  }  |   |   |
//    * |-------------------------------------------------------------------------+
//    * |       | F1  | F2  | F3  | F4  | F5  | F6  | F7  | F8  | F9  | F10 |     |
//    * |-------------------------------------------------------------------------+
//    * |     |     |     |Capsl|      |          |       | Home| PgDn| PgUp| End |
//    * `-------------------------------------------------------------------------'
   

// LAYOUT( /* Left */
//   KC_TILDE,  KC_EXCLAIM,  KC_AT,  KC_HASH,  KC_DOLLAR, KC_PERCENT, KC_CIRCUMFLEX, KC_AMPERSAND, KC_ASTERISK, KC_LEFT_PAREN, KC_RIGHT_PAREN, KC_DELETE,
//   _______, _______, _______, _______, _______, _______, _______, KC_UNDERSCORE, KC_PLUS, KC_LEFT_CURLY_BRACE, KC_RIGHT_CURLY_BRACE, KC_PIPE,
//   _______, KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5,   KC_F6,   KC_F7,   KC_F8,   KC_F9,   KC_F10,  _______,
//   _______, _______, _______, KC_CAPS, _______,     _______,      _______, KC_HOME, KC_PGDN, KC_PGUP, KC_END
// ),

//   /* FN Layer 2
//    * ,-------------------------------------------------------------------------.
//    * | Esc | Calc|Webhm| Mail| Comp|     |     |     |     |     |PrtSc|       |
//    * |-------------------------------------------------------------------------+
//    * |      |     |     |     |     |     |     |     |     |     |     |      |
//    * |-------------------------------------------------------------------------+
//    * |       |    |     |     |     |     |     |     |     |     |     |     |
//    * |-------------------------------------------------------------------------+
//    * |     |     |     |     |      |          |       |MousL|MousD|MousU|MousR|
//    * `-------------------------------------------------------------------------'
//    */

// LAYOUT( /* Tab */
//   KC_ESC,  KC_CALC, KC_WHOM, KC_MAIL, KC_MYCM, _______, _______, _______, _______, _______, KC_PSCR, _______,
//   _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
//   _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______, _______,
//   _______, _______, _______, _______, _______,     _______,      _______, KC_MS_L, KC_MS_D, KC_MS_U, KC_MS_R
// ),
// };
