#pragma once
#include "drashna.h"
/*
Since our quirky block definitions are basically a list of comma separated
arguments, we need a wrapper in order for these definitions to be
expanded before being used as arguments to the LAYOUT_xxx macro.
*/
#if (!defined(LAYOUT) && defined(KEYMAP))
#    define LAYOUT KEYMAP
#endif

// clang-format off
#define LAYOUT_ergodox_wrapper(...)          LAYOUT_ergodox(__VA_ARGS__)
#define LAYOUT_ergodox_pretty_wrapper(...)   LAYOUT_ergodox_pretty(__VA_ARGS__)
#define KEYMAP_wrapper(...)                  LAYOUT(__VA_ARGS__)
#define LAYOUT_wrapper(...)                  LAYOUT(__VA_ARGS__)
#define LAYOUT_ortho_4x12_wrapper(...)       LAYOUT_ortho_4x12(__VA_ARGS__)
#define LAYOUT_ortho_5x12_wrapper(...)       LAYOUT_ortho_5x12(__VA_ARGS__)
#define LAYOUT_gergo_wrapper(...)            LAYOUT_gergo(__VA_ARGS__)
#define LAYOUT_ut472_wrapper(...)            LAYOUT_ut472(__VA_ARGS__)
#define LAYOUT_dropALT_wrapper(...)          LAYOUT_dropALT(__VA_ARGS__)
#define LAYOUT_lily58_base_wrapper(...)      LAYOUT_lily58_base(__VA_ARGS__)

/*
Blocks for each of the four major keyboard layouts
Organized so we can quickly adapt and modify all of them
at once, rather than for each keyboard, one at a time.
And this allows for much cleaner blocks in the keymaps.
For instance Tap/Hold for Control on all of the layouts

NOTE: These are all the same length.  If you do a search/replace
  then you need to add/remove underscores to keep the
  lengths consistent.
*/

#define _________________QWERTY_L1_________________        KC_Q,    KC_W,    KC_E,    KC_R,    KC_T
#define _________________QWERTY_L2_________________        KC_A,    KC_S,    KC_D,    KC_F,    KC_G
#define _________________QWERTY_L3_________________        KC_Z,    KC_X,    KC_C,    KC_V,    KC_B

// #define _________________QWERTY_R1_________________        KC_Y,    KC_U,    KC_I,    KC_O,    TD_ae
// #define _________________QWERTY_R2_________________        KC_H,    KC_J,    KC_K,    KC_L,    TD_oe
// #define _________________QWERTY_R3_________________        KC_N,    KC_M,    KC_COMM, KC_DOT,  TD_aa

#define _________________QWERTY_R1_________________        KC_Y,    KC_U,    KC_I,    KC_O,    KC_P
#define _________________QWERTY_R2_________________        KC_H,    KC_J,    KC_K,    KC_L,    KC_SCLN
#define _________________QWERTY_R3_________________        KC_N,    KC_M,    KC_COMM, KC_DOT,  KC_SLASH

#define _________________WORKMAN_L1________________       KC_Q,    KC_D,    KC_R,   KC_W,     KC_B
#define _________________WORKMAN_L2________________       KC_A,    KC_S,    KC_H,   KC_T,     KC_G
#define _________________WORKMAN_L3________________       KC_Z,    KC_X,    KC_M,   KC_C,     KC_V

#define _________________WORKMAN_R1________________       KC_J,    KC_F,    KC_U,    KC_P,    KC_SCLN
#define _________________WORKMAN_R2________________       KC_Y,    KC_N,    KC_E,    KC_O,    KC_I
#define _________________WORKMAN_R3________________       KC_K,    KC_L,    KC_COMM, KC_DOT,  KC_SLASH

#define ________________NUMBER_LEFT________________       KC_1,    KC_2,    KC_3,    KC_4,    KC_5
#define ________________NUMBER_RIGHT_______________       KC_6,    KC_7,    KC_8,    KC_9,    KC_0
#define _________________FUNC_LEFT_________________       KC_F1,   KC_F2,   KC_F3,   KC_F4,   KC_F5
#define _________________FUNC_RIGHT________________       KC_F6,   KC_F7,   KC_F8,   KC_F9,   KC_F10

#define ___________________BLANK___________________        _______, _______, _______, _______, _______

#define _________________LOWER_L1__________________        ___________________BLANK___________________
#define _________________LOWER_L2__________________        _______, KC_MPRV, KC_MPLY, KC_MNXT, _______
#define _________________LOWER_L3__________________        ___________________BLANK___________________

#define _________________LOWER_R1__________________        KC_SLSH, KC_7,    KC_8,    KC_9, KC_MINS
#define _________________LOWER_R2__________________        KC_ASTR, KC_4,    KC_5,    KC_6, KC_COMM
#define _________________LOWER_R3__________________        KC_0,    KC_1,    KC_2,    KC_3, KC_EQL

#define _________________RAISE_L1__________________        KC_EXLM, KC_AT,   KC_LPRN, KC_RPRN, KC_PIPE
#define _________________RAISE_L2__________________        KC_HASH, KC_DLR,  KC_LBRC, KC_RBRC, KC_GRV
#define _________________RAISE_L3__________________        KC_PERC, KC_CIRC, KC_LCBR, KC_RCBR, KC_TILD

#define _________________RAISE_R1__________________        _______, KC_UNDS, ALGR(KC_5),_______,KC_BSLS
#define _________________RAISE_R2__________________        KC_PLUS, KC_MINS, KC_SLSH, KC_ASTR, KC_PERC
#define _________________RAISE_R3__________________        KC_AMPR, KC_EQL,  KC_COMM, KC_DOT,  KC_SLSH

#define _________________ADJUST_L1_________________        ___________________BLANK___________________
#define _________________ADJUST_L2_________________        _______, RGB_SAI, RGB_HUI, RGB_VAI, _______
#define _________________ADJUST_L3_________________        _______, RGB_SAD, RGB_HUD, RGB_VAD, _______

#define _________________ADJUST_R1_________________        RGB_TOG, KC_F7,   KC_F8,   KC_F9,   KC_F10
#define _________________ADJUST_R2_________________        _______, KC_F4,   KC_F5,   KC_F6,   KC_F11
#define _________________ADJUST_R3_________________        _______, KC_F1,   KC_F2,   KC_F3,   KC_F12

#define ___________________NAV_L1__________________        DF(_QWERTY), DF(_WORKMAN), TG(_WARCRAFT), TG(_DIABLO), TG(_HEROES)      
#define ___________________NAV_L2__________________        _______, _______, KC_UP  , _______, _______
#define ___________________NAV_L3__________________        _______, KC_LEFT, KC_DOWN, KC_RGHT, _______

#define ___________________NAV_R1__________________        KC_PGUP, KC_HOME, KC_UP,   KC_END,  aa
#define ___________________NAV_R2__________________        KC_PGDN, KC_LEFT, KC_DOWN, KC_RGHT, ae
#define ___________________NAV_R3__________________        _______, _______, _______, _______, oe


// clang-format on